package br.inf.mfp.gestaoadministrativa.app;

import br.inf.mfp.gestaoadministrativa.entity.*;
import io.jmix.core.DataManager;
import io.jmix.core.SaveContext;
import io.jmix.core.security.CurrentAuthentication;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component("ga_GrupoEconomicoBean")
public class GrupoEconomicoBean {
    private static final Logger log = LoggerFactory.getLogger(GrupoEconomicoBean.class);

    @Autowired
    private DataManager dataManager;

    @Autowired
    private CurrentAuthentication usuarioAut;

//    public String loginUsuario;
    @Autowired
    private CurrentAuthentication currentAuthentication;

    public List<UsuarioEmpresa> CargaEmpresaUsuario() throws gaException {

        //if (loginUsuario == null || loginUsuario.length() == 0)
        String    loginUsuario = usuarioAut.getUser().getUsername();

        List<UsuarioEmpresa> listaRetorno = new ArrayList<>();

        List<UsuarioEmpresa> listaUE = dataManager.load(UsuarioEmpresa.class)
                .query("e.usuario.username = :login  ORDER BY e.empresa.codigo")
                .parameter("login", loginUsuario)
                .list();

        List<Empresa> listaEmp = dataManager.load(Empresa.class)
                .query("e.situacao = 10 ORDER BY e.codigo")
                .list();

        if (listaEmp.size() == 0) {
            throw new gaException("Não retornou nenhuma empresa para associação");
        }

        listaEmp.forEach(empresa -> {

            boolean achou = false;

            for (UsuarioEmpresa ue : listaUE) {
                if (ue.getEmpresa().getId().equals(empresa.getId())) {
                    achou = true;
                    if (ue.getSituacao() == null)
                        ue.setSituacao(Situacao.ATIVO);

                    listaRetorno.add(ue);
                    break;
                }

            }

            if (!achou) {
                UsuarioEmpresa ue = dataManager.create(UsuarioEmpresa.class);
                ue.setEmpresa(empresa);
                ue.setSituacao(Situacao.BLOQUEADO);
                listaRetorno.add(ue);
            }

        });


        return listaRetorno;
    }

    public void SalvarUsuariosEmpresa(List<UsuarioEmpresa> itens) {
        if (itens != null) {

            //if (loginUsuario == null || loginUsuario.length() == 0)
             String   loginUsuario = usuarioAut.getUser().getUsername();

            User usuario = dataManager.load(User.class)
                    .query("e.username = :login")
                    .parameter("login", loginUsuario)
                    .one();

            SaveContext listaSalvar = new SaveContext();

            for (UsuarioEmpresa ue : itens) {
                if (ue.getUsuario() == null) {
                    ue.setUsuario(usuario);
                }
                listaSalvar.saving(ue);
            }

            if (listaSalvar.getEntitiesToSave().size() > 0) {
                dataManager.save(listaSalvar);
            }

        }

    }

    public GrupoEconomico retornarGE() throws gaException {

        //if (loginUsuario == null || loginUsuario.length() == 0)
        String    loginUsuario = usuarioAut.getUser().getUsername();

        UserDetails user = currentAuthentication.getUser();


        log.trace("Usuaario" + user.getUsername());

        User ge = dataManager.load(User.class)
                .query("e.username = :login ")
                .parameter("login", loginUsuario)
                .one();

        return ge.getGrupoEconomico();

    }
    public List<Empresa> retornarEmpresas() throws gaException {

        //if (loginUsuario == null || loginUsuario.length() == 0)
        String    loginUsuario = usuarioAut.getUser().getUsername();

        UserDetails user = currentAuthentication.getUser();


        log.trace("Usuaario" + user.getUsername());

        User ge = dataManager.load(User.class)
                .query("e.username = :login ")
                .parameter("login", loginUsuario)
                .one();



    List<Empresa> lista = new ArrayList<>();
    ge.getEmpresas().forEach(usuarioEmpresa -> {
        lista.add(usuarioEmpresa.getEmpresa());
    });

        return lista;

    }



}