package br.inf.mfp.gestaoadministrativa.app;

import br.inf.mfp.gestaoadministrativa.entity.SituacaoTraducao;
import br.inf.mfp.gestaoadministrativa.entity.Traducao;
import br.inf.mfp.gestaoadministrativa.entity.TraducaoLinhas;
import io.jmix.core.*;
import liquibase.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.*;
import java.util.*;

import com.google.cloud.translate.*;

@Component("ga_TraducaoBean")
public class TraducaoBean {
    private static final Logger log = LoggerFactory.getLogger(TraducaoBean.class);
    @Autowired
    private DataManager dataManager;
    @Autowired
    private FileStorageLocator fileStorageLocator;

    public boolean Carregar(UUID chave) throws Exception {

        try {

            if (chave == null)
                throw new gaException("Deve ser informada uma chave");


            Traducao tr = dataManager.load(Traducao.class)
                    .query("e.id = :chave")
                    .parameter("chave", chave)
                    .one();

            if (tr.getArquivoOriginal() == null) {
                throw new gaException("Não consta nenhum arquivo original");
            }

            FileRef arquivo = tr.getArquivoOriginal();

            log.trace("arquivo -> " + arquivo);
            FileStorage local = fileStorageLocator.getDefault();
            InputStream teste = local.openStream(arquivo);
            BufferedReader conteudo = new BufferedReader(new InputStreamReader(teste));
            //Scanner conteudo = new Scanner(arquivo.getPath());

            String linha = conteudo.readLine();
            StringBuilder sbC = new StringBuilder();
            StringBuilder sbL = new StringBuilder();
            String chaveL = "";
            String valor;
            boolean copiar = false;

            while (linha != null) {
                if (!linha.startsWith("#")) {
                    log.info(linha);


                    if (linha.startsWith("  ")) {
                        sbL.append("<rn>");

                        int numEspaco = 0;
                        while (linha.charAt(numEspaco) == ' ') {
                            numEspaco++;
                        }

                        linha = "<l0" + (numEspaco) + ">" + linha.substring(numEspaco);
                        sbL.append(linha);

                    } else if (linha.indexOf("=") > 0) {

                        if (!sbL.isEmpty()) {
                            log.info("valor -> " + sbL);
                            ValidarLinha(tr, chaveL, sbL.toString(), copiar);
                            sbL = new StringBuilder();
                        }

                        chaveL = linha.substring(0, linha.indexOf("="));
                        valor = linha.substring(linha.indexOf("=") + 1);

                        copiar = true;
                        int posicao = 0;

                        while (posicao < valor.length()) {

                            if (Character.isLetter(valor.charAt(posicao))) {
                                copiar = false;
                                break;
                            }
                            posicao++;
                        }

                        sbL.append(valor);
                        log.info(chaveL + " - " + valor);
                        log.info("conteudo ->" + sbL);

                    }

                } else {
                    sbC.append(linha);
                    sbC.append("<rn>");
                }
                linha = conteudo.readLine();
            }

            if (!sbL.isEmpty()) {
                ValidarLinha(tr, chaveL, sbL.toString(), copiar);
            }


            if (sbC.toString().length() > 0) {
                tr.setCabecalho(sbC.toString());
                dataManager.save(tr);
            }


        } catch (Exception ee) {
            log.error(ee.getMessage());
            throw new Exception(ee.getMessage());
        }

        return true;
    }

    private void ValidarLinha(Traducao tr, String chaveL, String valor, boolean copiarTr) {

        List<TraducaoLinhas> reg = dataManager.load(TraducaoLinhas.class)
                .query("e.traducao = :tr AND e.chave = :chave")
                .parameter("tr", tr)
                .parameter("chave", chaveL)
                .list();

        if (reg.size() == 0) {


            List<TraducaoLinhas> regs = dataManager.load(TraducaoLinhas.class)
                    .query("e.traducao = :tr")
                    .parameter("tr", tr)
                    .list();

            int numRegistro = regs.size() + 1;


            TraducaoLinhas novo = dataManager.create(TraducaoLinhas.class);
            novo.setTraducao(tr);
            novo.setChave(chaveL);
            novo.setConteudoOriginal(valor);
            novo.setOrdem(numRegistro);
            if (copiarTr) {
                novo.setConteudoTraduzido(valor);
                novo.setSituacao(SituacaoTraducao.COPIADO);
            }
            dataManager.save(novo);

        }

    }

    public String ChamarTraducao(String conteudo) {

        log.info(String.format("resultado [ %s ] ", conteudo));

        String valor = conteudo.replace("\r\n", "<rn>");

        Translate translate = TranslateOptions.getDefaultInstance().getService();

        Translation translation = translate.translate(
                valor,
                Translate.TranslateOption.sourceLanguage("en"),
                Translate.TranslateOption.targetLanguage("pt"),
                Translate.TranslateOption.model("nmt"));

        if (translation.getTranslatedText() != null) {
            String retorno = translation.getTranslatedText();

            retorno = AjusteRetorno(retorno);
            log.info(String.format("resultado [ %s ] ", retorno));
            return retorno;
        }
        return "";
    }

    private String AjusteRetorno(String retorno) {

        String valor = retorno;

        valor = valor.replace("&#39;", "'");
        valor = valor.replace("&quot;", "\"");
        valor = valor.replace("&lt;", "<");
        valor = valor.replace("&gt;", ">");

        log.debug(String.format("correcao -> [%s]", valor));

        return valor;
    }

    public List<String> ChamarTraducao(List<String> conteudo) throws Exception {

        try {
            Translate translate = TranslateOptions.getDefaultInstance().getService();

            List<Translation> traducao = translate.translate(
                    conteudo,
                    Translate.TranslateOption.sourceLanguage("en"),
                    Translate.TranslateOption.targetLanguage("pt"),
                    Translate.TranslateOption.model("nmt"));

            List<String> retorno = new LinkedList<>();
            traducao.forEach(translation -> {
                String retornoTxt = translation.getTranslatedText();
                retornoTxt = AjusteRetorno(retornoTxt);
                retorno.add(retornoTxt);
            });
            return retorno;

        } catch (Exception ee) {
            log.error(ee.getMessage());
            throw new Exception(ee.getMessage());
        }

    }

    public void TraduzirArq(UUID chave) throws Exception {
        try {

            if (chave == null)
                throw new gaException("Deve ser informada uma chave");


            Traducao tr = dataManager.load(Traducao.class)
                    .query("e.id = :chave")
                    .parameter("chave", chave)
                    .one();

            if (tr.getTraducaoLinhas() == null || tr.getTraducaoLinhas().size() == 0) {
                throw new gaException("Nenhuma linha de tradução");
            }

            List<TraducaoLinhas> linhas = new ArrayList<>();

            for (TraducaoLinhas tl : tr.getTraducaoLinhas()) {
                if (tl.getSituacao().equals(SituacaoTraducao.PENDENTE) && tl.getConteudoOriginal().length() > 1) {
                    linhas.add(tl);
                    if (linhas.size() > 10) {

                        ProcessarTraducaoLinhas(linhas);
                        linhas = new ArrayList<>();
                    }

                }

            }
            if (linhas.size() > 0) {

                ProcessarTraducaoLinhas(linhas);
            }

        } catch (Exception ee) {
            log.error(ee.getMessage());
            throw new Exception(ee.getMessage());
        }

    }

    private void ProcessarTraducaoLinhas(List<TraducaoLinhas> linhas) throws Exception {

        try {
            List<String> chavesTr = new ArrayList<>();
            for (TraducaoLinhas traducaoLinhas : linhas) {
                chavesTr.add(traducaoLinhas.getConteudoOriginal());
            }

            List<String> retorno = ChamarTraducao(chavesTr);

            SaveContext listaSalvar = new SaveContext();
            if (retorno.size() > 0) {

                for (int i = 0; i < retorno.size(); i++) {

                    TraducaoLinhas trU = linhas.get(i);
                    trU.setRetornoApi(retorno.get(i));
                    trU.setSituacao(SituacaoTraducao.TRADUZIDO_API);
                    trU.setConteudoTraduzido(retorno.get(i));
                    listaSalvar.saving(trU);

                }

                dataManager.save(listaSalvar);

            }

        } catch (Exception ee) {

            log.error(ee.getMessage());
            throw new gaException(ee.getMessage());

        }

    }

    public ClasseGenerica SalvarArquivo(UUID chave) {
        ClasseGenerica retorno = new ClasseGenerica();
        try {
            if (chave == null)
                throw new gaException("Deve ser informada uma chave");


            Traducao tr = dataManager.load(Traducao.class)
                    .query("e.id = :chave")
                    .parameter("chave", chave)
                    .one();

            if (tr.getTraducaoLinhas() == null || tr.getTraducaoLinhas().size() == 0) {
                throw new gaException("Nenhuma linha de tradução");
            }


            StringBuilder sb = new StringBuilder();

            retorno.setNomeArquivo(tr.getPastaOrigem() + "-messages_pt_BR.properties");

            tr.getTraducaoLinhas().forEach(lin -> {

                if (!lin.getSituacao().equals(SituacaoTraducao.PENDENTE)) {
                    sb.append(lin.getChave());
                    sb.append(" = ");


                    if (lin.getConteudoTraduzido().indexOf("<l0") > 0) {

                        StringBuilder tratar = new StringBuilder();
                        tratar.append(lin.getConteudoTraduzido());

                        String conteudoTr = ProcessarEspaco(tratar);

                        sb.append(conteudoTr.replace("<rn>", "\r\n"));

                    } else {
                        sb.append(lin.getConteudoTraduzido());
                    }

                    sb.append("\r\n");
                }
            });
            retorno.setLinhas(sb);


        } catch (Exception ee) {
            log.error(ee.getMessage());
        }
        return retorno;
    }

    private String ProcessarEspaco(StringBuilder conteudoTraduzido) {

        int posicao;
        String dadosTr = conteudoTraduzido.toString();

        posicao = dadosTr.indexOf("<l0");

        int valor = Integer.parseInt(dadosTr.substring(posicao + 2, dadosTr.indexOf(">", posicao)));

        log.info("achou o valor de " + valor);
        dadosTr = dadosTr.replace("<l0" + valor + ">", StringUtil.repeat(" ", valor));

        if (dadosTr.indexOf("<l0") > 0)
            dadosTr = ProcessarEspaco(new StringBuilder(dadosTr));

        return dadosTr;
    }
}