package br.inf.mfp.gestaoadministrativa.entity;

import io.jmix.core.DeletePolicy;
import io.jmix.core.annotation.DeletedBy;
import io.jmix.core.annotation.DeletedDate;
import io.jmix.core.entity.annotation.JmixGeneratedValue;
import io.jmix.core.entity.annotation.OnDelete;
import io.jmix.core.metamodel.annotation.*;
import org.springframework.data.annotation.CreatedBy;
import org.springframework.data.annotation.CreatedDate;
import org.springframework.data.annotation.LastModifiedBy;
import org.springframework.data.annotation.LastModifiedDate;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@JmixEntity
@Table(name = "CONTRATO", indexes = {
        @Index(name = "IDX_GA_CONTRATO_PESSOA", columnList = "PESSOA_ID"),
        @Index(name = "IDX_CONTRATO_GRUPO_ECONOMICO", columnList = "GRUPO_ECONOMICO_ID"),
        @Index(name = "IDX_CONTRATO_EMPRESA_FATURAMENTO", columnList = "EMPRESA_FATURAMENTO_ID")
})
@Entity(name = "ga_Contrato")
public class Contrato {
    @JmixGeneratedValue
    @Column(name = "ID", nullable = false)
    @Id
    private UUID id;

    @Column(name = "VERSION", nullable = false)
    @Version
    private Integer version;

    @CreatedBy
    @Column(name = "CREATED_BY")
    private String createdBy;

    @CreatedDate
    @Column(name = "CREATED_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date createdDate;

    @LastModifiedBy
    @Column(name = "LAST_MODIFIED_BY")
    private String lastModifiedBy;

    @LastModifiedDate
    @Column(name = "LAST_MODIFIED_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date lastModifiedDate;

    @DeletedBy
    @Column(name = "DELETED_BY")
    private String deletedBy;

    @DeletedDate
    @Column(name = "DELETED_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date deletedDate;

    @JoinColumn(name = "GRUPO_ECONOMICO_ID", nullable = false)
    @NotNull
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    private GrupoEconomico grupoEconomico;

    @JoinColumn(name = "EMPRESA_FATURAMENTO_ID", nullable = false)
    @NotNull
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    private Empresa empresaFaturamento;

    @JoinColumn(name = "PESSOA_ID", nullable = false)
    @NotNull
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    private Pessoa pessoa;

    @Column(name = "SITUACAO", nullable = false)
    @NotNull
    private Integer situacao = Situacao.ATIVO.getId();

    @Column(name = "TIPO_CONTRATO", nullable = false)
    @NotNull
    private Integer tipoContrato = TipoContrato.PAGAR.getId();

    @Column(name = "TIPO_FATURAMENTO", nullable = false)
    @NotNull
    private Integer tipoFaturamento = TipoFaturamento.MENSAL.getId();

    @NumberFormat(pattern = "#0")
    @Column(name = "DIA_FATURAMENTO", nullable = false)
    @NotNull
    private Integer diaFaturamento = 1;

    @Column(name = "DATA_VENCIMENTO", nullable = false)
    @Temporal(TemporalType.DATE)
    @NotNull
    private Date dataVencimento;

    @Column(name = "OBSERVACAO_INTERNA")
    private String observacaoInterna;

    @Column(name = "INFORMACAO_ADICIONAL")
    private String informacaoAdicional;

    @Column(name = "TIPO_DESCONTO", nullable = false)
    @NotNull
    private Integer tipoDesconto = TipoValor.PERCENTUAL.getId();

    @NumberFormat(pattern = "###,##0.00")
    @Column(name = "PERCENTUAL_DESCONTO", nullable = false, precision = 19, scale = 2)
    @NotNull
    private BigDecimal percentualDesconto = BigDecimal.ZERO;

    @NumberFormat(pattern = "###,##0.00")
    @Column(name = "VALOR_DESCONTO", nullable = false, precision = 19, scale = 2)
    @NotNull
    private BigDecimal valorDesconto = BigDecimal.ZERO;

    @NumberFormat(pattern = "###,##0.00")
    @Column(name = "PERCENTUAL_JUROS", nullable = false, precision = 19, scale = 2)
    @NotNull
    private BigDecimal percentualJuros = BigDecimal.ZERO;

    @NumberFormat(pattern = "###,##0.00")
    @NotNull
    @Column(name = "PERCENTUAL_MULTA", nullable = false, precision = 19, scale = 2)
    private BigDecimal percentualMulta = BigDecimal.ZERO;

    @NumberFormat(pattern = "###,##0.00")
    @Column(name = "VALOR", nullable = false, precision = 19, scale = 2)
    @NotNull
    private BigDecimal valor = BigDecimal.ZERO;

    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    @OneToMany(mappedBy = "contrato")
    private List<ContratoTitulo> contratoTitulo;

    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    @OneToMany(mappedBy = "contrato")
    private List<ContratoLog> contratoLog;

    public Empresa getEmpresaFaturamento() {
        return empresaFaturamento;
    }

    public void setEmpresaFaturamento(Empresa empresaFaturamento) {
        this.empresaFaturamento = empresaFaturamento;
    }

    public GrupoEconomico getGrupoEconomico() {
        return grupoEconomico;
    }

    public void setGrupoEconomico(GrupoEconomico grupoEconomico) {
        this.grupoEconomico = grupoEconomico;
    }

    public Date getDataVencimento() {
        return dataVencimento;
    }

    public void setDataVencimento(Date dataVencimento) {
        this.dataVencimento = dataVencimento;
    }

    public List<ContratoLog> getContratoLog() {
        return contratoLog;
    }

    public void setContratoLog(List<ContratoLog> contratoLog) {
        this.contratoLog = contratoLog;
    }

    public List<ContratoTitulo> getContratoTitulo() {
        return contratoTitulo;
    }

    public void setContratoTitulo(List<ContratoTitulo> contratoTitulo) {
        this.contratoTitulo = contratoTitulo;
    }

    public BigDecimal getValor() {
        return valor;
    }

    public void setValor(BigDecimal valor) {
        this.valor = valor;
    }

    public BigDecimal getPercentualMulta() {
        return percentualMulta;
    }

    public void setPercentualMulta(BigDecimal percentualMulta) {
        this.percentualMulta = percentualMulta;
    }

    public BigDecimal getPercentualJuros() {
        return percentualJuros;
    }

    public void setPercentualJuros(BigDecimal percentualJuros) {
        this.percentualJuros = percentualJuros;
    }

    public TipoValor getTipoDesconto() {
        return tipoDesconto == null ? null : TipoValor.fromId(tipoDesconto);
    }

    public void setTipoDesconto(TipoValor tipoDesconto) {
        this.tipoDesconto = tipoDesconto == null ? null : tipoDesconto.getId();
    }

    public BigDecimal getValorDesconto() {
        return valorDesconto;
    }

    public void setValorDesconto(BigDecimal valorDesconto) {
        this.valorDesconto = valorDesconto;
    }

    public BigDecimal getPercentualDesconto() {
        return percentualDesconto;
    }

    public void setPercentualDesconto(BigDecimal percentualDesconto) {
        this.percentualDesconto = percentualDesconto;
    }

    public String getInformacaoAdicional() {
        return informacaoAdicional;
    }

    public void setInformacaoAdicional(String informacaoAdicional) {
        this.informacaoAdicional = informacaoAdicional;
    }

    public String getObservacaoInterna() {
        return observacaoInterna;
    }

    public void setObservacaoInterna(String observacaoInterna) {
        this.observacaoInterna = observacaoInterna;
    }

    public Integer getDiaFaturamento() {
        return diaFaturamento;
    }

    public void setDiaFaturamento(Integer diaFaturamento) {
        this.diaFaturamento = diaFaturamento;
    }

    public TipoFaturamento getTipoFaturamento() {
        return tipoFaturamento == null ? null : TipoFaturamento.fromId(tipoFaturamento);
    }

    public void setTipoFaturamento(TipoFaturamento tipoFaturamento) {
        this.tipoFaturamento = tipoFaturamento == null ? null : tipoFaturamento.getId();
    }

    public TipoContrato getTipoContrato() {
        return tipoContrato == null ? null : TipoContrato.fromId(tipoContrato);
    }

    public void setTipoContrato(TipoContrato tipoContrato) {
        this.tipoContrato = tipoContrato == null ? null : tipoContrato.getId();
    }

    public Situacao getSituacao() {
        return situacao == null ? null : Situacao.fromId(situacao);
    }

    public void setSituacao(Situacao situacao) {
        this.situacao = situacao == null ? null : situacao.getId();
    }

    public Pessoa getPessoa() {
        return pessoa;
    }

    public void setPessoa(Pessoa pessoa) {
        this.pessoa = pessoa;
    }

    public Date getDeletedDate() {
        return deletedDate;
    }

    public void setDeletedDate(Date deletedDate) {
        this.deletedDate = deletedDate;
    }

    public String getDeletedBy() {
        return deletedBy;
    }

    public void setDeletedBy(String deletedBy) {
        this.deletedBy = deletedBy;
    }

    public Date getLastModifiedDate() {
        return lastModifiedDate;
    }

    public void setLastModifiedDate(Date lastModifiedDate) {
        this.lastModifiedDate = lastModifiedDate;
    }

    public String getLastModifiedBy() {
        return lastModifiedBy;
    }

    public void setLastModifiedBy(String lastModifiedBy) {
        this.lastModifiedBy = lastModifiedBy;
    }

    public Date getCreatedDate() {
        return createdDate;
    }

    public void setCreatedDate(Date createdDate) {
        this.createdDate = createdDate;
    }

    public String getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(String createdBy) {
        this.createdBy = createdBy;
    }

    public Integer getVersion() {
        return version;
    }

    public void setVersion(Integer version) {
        this.version = version;
    }

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }

    @InstanceName
    @DependsOnProperties({"pessoa", "tipoContrato", "situacao"})
    public String getInstanceName() {
        return String.format("%s %s %s", pessoa, tipoContrato, situacao);
    }
}