package br.inf.mfp.gestaoadministrativa.entity;

import io.jmix.core.DeletePolicy;
import io.jmix.core.annotation.DeletedBy;
import io.jmix.core.annotation.DeletedDate;
import io.jmix.core.entity.annotation.JmixGeneratedValue;
import io.jmix.core.entity.annotation.OnDelete;
import io.jmix.core.metamodel.annotation.*;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.data.annotation.CreatedBy;
import org.springframework.data.annotation.CreatedDate;
import org.springframework.data.annotation.LastModifiedBy;
import org.springframework.data.annotation.LastModifiedDate;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@JmixEntity
@Table(name = "PEDIDO", indexes = {
        @Index(name = "IDX_GA_PEDIDO_FORNECEDOR", columnList = "FORNECEDOR_ID"),
        @Index(name = "IDX_GA_PEDIDO_GRUPO_ECONOMICO", columnList = "GRUPO_ECONOMICO_ID"),
        @Index(name = "IDX_GA_PEDIDO_EMPRESA", columnList = "EMPRESA_ID"),
        @Index(name = "IDX_GA_PEDIDO_UNQ", columnList = "NUMERO", unique = true),
        @Index(name = "IDX_PEDIDO_TITULO_PAGAR", columnList = "TITULO_PAGAR_ID"),
        @Index(name = "IDX_PEDIDO_TITULO_RECEBER", columnList = "TITULO_RECEBER_ID")
})
@Entity(name = "ga_Pedido")
public class Pedido {


    @JmixGeneratedValue
    @Column(name = "ID", nullable = false)
    @Id
    private UUID id;

    @Column(name = "VERSION", nullable = false)
    @Version
    private Integer version;

    @CreatedBy
    @Column(name = "CREATED_BY")
    private String createdBy;

    @CreatedDate
    @Column(name = "CREATED_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date createdDate;

    @LastModifiedBy
    @Column(name = "LAST_MODIFIED_BY")
    private String lastModifiedBy;

    @LastModifiedDate
    @Column(name = "LAST_MODIFIED_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date lastModifiedDate;

    @DeletedBy
    @Column(name = "DELETED_BY")
    private String deletedBy;

    @DeletedDate
    @Column(name = "DELETED_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date deletedDate;

    @JoinColumn(name = "GRUPO_ECONOMICO_ID", nullable = false)
    @NotNull
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    private GrupoEconomico grupoEconomico;

    @JoinColumn(name = "EMPRESA_ID", nullable = false)
    @NotNull
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    private Empresa empresa;


    @NumberFormat(pattern = "###,##0")
    @Column(name = "NUMERO", nullable = false)
    @NotNull
    private Integer numero;

    @JoinColumn(name = "FORNECEDOR_ID", nullable = false)
    @NotNull
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    private Pessoa fornecedor;

    @Column(name = "DATA_PEDIDO", nullable = false)
    @Temporal(TemporalType.DATE)
    @NotNull
    private Date dataPedido = new Date();

    @Column(name = "ENTREGA", nullable = false)
    @Temporal(TemporalType.DATE)
    @NotNull
    private Date entrega = new Date();

    @Column(name = "PERIODO", nullable = false)
    @NotNull
    private Integer periodo = Periodo.TARDE.getId();

    @Column(name = "SITUACAO_PEDIDO", nullable = false)
    @NotNull
    private Integer situacaoPedido = SituacaoPedido.PENDENTE.getId();

    @NumberFormat(pattern = "###,##0.00", decimalSeparator = ",", groupingSeparator = ".")
    @Column(name = "VALOR_TOTAL", nullable = false, precision = 19, scale = 2)
    @NotNull
    private BigDecimal valorTotal = BigDecimal.ZERO;

    @NumberFormat(pattern = "###,##0.00", decimalSeparator = ",", groupingSeparator = ".")
    @Column(name = "VALOR_CREDITO", precision = 19, scale = 2)
    private BigDecimal valorCredito = BigDecimal.ZERO;

    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    @OneToMany(mappedBy = "pedido")
    private List<ItemPedido> itemPedido;

    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    @OneToMany(mappedBy = "pedido")
    private List<ParcelaPedido> parcelaPedido;

    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    @OrderBy("item ASC")
    @OneToMany(mappedBy = "pedido")
    private List<RetornoPedido> retornoPedido;

    @JoinColumn(name = "TITULO_PAGAR_ID")
    @OneToOne(fetch = FetchType.LAZY)
    private TituloPagar tituloPagar;

    @JoinColumn(name = "TITULO_RECEBER_ID")
    @OneToOne(fetch = FetchType.LAZY)
    private TituloReceber tituloReceber;

    public TituloReceber getTituloReceber() {
        return tituloReceber;
    }

    public void setTituloReceber(TituloReceber tituloReceber) {
        this.tituloReceber = tituloReceber;
    }

    public TituloPagar getTituloPagar() {
        return tituloPagar;
    }

    public void setTituloPagar(TituloPagar tituloPagar) {
        this.tituloPagar = tituloPagar;
    }

    public List<ParcelaPedido> getParcelaPedido() {
        return parcelaPedido;
    }

    public void setParcelaPedido(List<ParcelaPedido> parcelaPedido) {
        this.parcelaPedido = parcelaPedido;
    }

    public List<RetornoPedido> getRetornoPedido() {
        return retornoPedido;
    }

    public void setRetornoPedido(List<RetornoPedido> retornoPedido) {
        this.retornoPedido = retornoPedido;
    }

    public List<ItemPedido> getItemPedido() {
        return itemPedido;
    }

    public void setItemPedido(List<ItemPedido> itemPedido) {
        this.itemPedido = itemPedido;
    }

    public Empresa getEmpresa() {
        return empresa;
    }

    public void setEmpresa(Empresa empresa) {
        this.empresa = empresa;
    }

    public GrupoEconomico getGrupoEconomico() {
        return grupoEconomico;
    }

    public void setGrupoEconomico(GrupoEconomico grupoEconomico) {
        this.grupoEconomico = grupoEconomico;
    }

    public Pessoa getFornecedor() {
        return fornecedor;
    }

    public void setFornecedor(Pessoa fornecedor) {
        this.fornecedor = fornecedor;
    }

    public BigDecimal getValorCredito() {
        return valorCredito;
    }

    public void setValorCredito(BigDecimal valorCredito) {
        this.valorCredito = valorCredito;
    }

    public BigDecimal getValorTotal() {
        return valorTotal;
    }

    public void setValorTotal(BigDecimal valorTotal) {
        this.valorTotal = valorTotal;
    }

    public SituacaoPedido getSituacaoPedido() {
        return situacaoPedido == null ? null : SituacaoPedido.fromId(situacaoPedido);
    }

    public void setSituacaoPedido(SituacaoPedido situacaoPedido) {
        this.situacaoPedido = situacaoPedido == null ? null : situacaoPedido.getId();
    }

    public Periodo getPeriodo() {
        return periodo == null ? null : Periodo.fromId(periodo);
    }

    public void setPeriodo(Periodo periodo) {
        this.periodo = periodo == null ? null : periodo.getId();
    }

    public Date getEntrega() {
        return entrega;
    }

    public void setEntrega(Date entrega) {
        this.entrega = entrega;
    }

    public Date getDataPedido() {
        return dataPedido;
    }

    public void setDataPedido(Date dataPedido) {
        this.dataPedido = dataPedido;
    }



    public Integer getNumero() {
        return numero;
    }

    public void setNumero(Integer numero) {
        this.numero = numero;
    }

    public Date getDeletedDate() {
        return deletedDate;
    }

    public void setDeletedDate(Date deletedDate) {
        this.deletedDate = deletedDate;
    }

    public String getDeletedBy() {
        return deletedBy;
    }

    public void setDeletedBy(String deletedBy) {
        this.deletedBy = deletedBy;
    }

    public Date getLastModifiedDate() {
        return lastModifiedDate;
    }

    public void setLastModifiedDate(Date lastModifiedDate) {
        this.lastModifiedDate = lastModifiedDate;
    }

    public String getLastModifiedBy() {
        return lastModifiedBy;
    }

    public void setLastModifiedBy(String lastModifiedBy) {
        this.lastModifiedBy = lastModifiedBy;
    }

    public Date getCreatedDate() {
        return createdDate;
    }

    public void setCreatedDate(Date createdDate) {
        this.createdDate = createdDate;
    }

    public String getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(String createdBy) {
        this.createdBy = createdBy;
    }

    public Integer getVersion() {
        return version;
    }

    public void setVersion(Integer version) {
        this.version = version;
    }

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }

    @InstanceName
    @DependsOnProperties({"numero", "entrega", "fornecedor"})
    public String getInstanceName() {
        return String.format("%s - %s - %s-%s", numero, DateFormatUtils.format(entrega, "dd/MM/yyyy" ) , fornecedor.getCodigo(), fornecedor.getNome());
    }
}