package br.inf.mfp.gestaoadministrativa.entity;

import io.jmix.core.entity.annotation.JmixGeneratedValue;
import io.jmix.core.metamodel.annotation.JmixEntity;
import io.jmix.core.metamodel.annotation.JmixProperty;
import io.jmix.core.metamodel.annotation.NumberFormat;
import org.springframework.data.annotation.CreatedBy;
import org.springframework.data.annotation.CreatedDate;
import org.springframework.data.annotation.LastModifiedBy;
import org.springframework.data.annotation.LastModifiedDate;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;
import java.util.UUID;

@JmixEntity
@Table(name = "RETORNO_PEDIDO", indexes = {
        @Index(name = "IDX_GA_RETORNO_PEDIDO_PEDIDO", columnList = "PEDIDO_ID"),
        @Index(name = "IDX_GA_RETORNO_PEDIDO_ITEM", columnList = "ITEM_ID"),
        @Index(name = "IDX_GA_RETORNO_PEDIDO_UNQ", columnList = "PEDIDO_ID, ITEM_ID", unique = true),
        @Index(name = "IDX_RETORNO_PEDIDO_MOVIMENTO_ESTOQUE", columnList = "MOVIMENTO_ESTOQUE_ID"),
        @Index(name = "IDX_RETORNO_PEDIDO_ITEM_PEDIDO", columnList = "ITEM_PEDIDO_ID")
})
@Entity(name = "ga_RetornoPedido")
public class RetornoPedido {
    @JmixGeneratedValue
    @Column(name = "ID", nullable = false)
    @Id
    private UUID id;

    @Column(name = "VERSION", nullable = false)
    @Version
    private Integer version;

    @CreatedBy
    @Column(name = "CREATED_BY")
    private String createdBy;

    @CreatedDate
    @Column(name = "CREATED_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date createdDate;

    @LastModifiedBy
    @Column(name = "LAST_MODIFIED_BY")
    private String lastModifiedBy;

    @LastModifiedDate
    @Column(name = "LAST_MODIFIED_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date lastModifiedDate;

    @JoinColumn(name = "PEDIDO_ID", nullable = false)
    @NotNull
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    private Pedido pedido;

    @JoinColumn(name = "ITEM_ID", nullable = false)
    @NotNull
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    private Item item;

    @NotNull
    @NumberFormat(pattern = "###,##0.000", decimalSeparator = ",", groupingSeparator = ".")
    @Column(name = "QUANTIDADE_INFORMADA", nullable = false)
    private Double quantidadeInformada;

    @Column(name = "NAO_ENTREGUE")
    private Boolean naoEntregue = false;

    @JoinColumn(name = "MOVIMENTO_ESTOQUE_ID")
    @OneToOne(fetch = FetchType.LAZY)
    private MovimentoEstoque movimentoEstoque;

    @Column(name = "SITUACAO_REGISTRO")
    private Integer situacaoRegistro = SituacaoRegistro.PENDENTE.getId();

    @NumberFormat(pattern = "###,##0.000")
    @Column(name = "VALOR_UNITARIO", precision = 19, scale = 2)
    private BigDecimal valorUnitario = BigDecimal.ZERO;

    @NumberFormat(pattern = "###,##0.00")
    @Column(name = "VALOR_TOTAL", precision = 19, scale = 2)
    private BigDecimal valorTotal = BigDecimal.ZERO;

    @JoinColumn(name = "ITEM_PEDIDO_ID")
    @OneToOne(fetch = FetchType.LAZY)
    private ItemPedido itemPedido;

    @JmixProperty
    public Double getQtdDiferenca() {

        if (getItemPedido() != null){
            return getQuantidadeInformada() - getItemPedido().getQuantidade();
        }
        return 0D;
    }

    public ItemPedido getItemPedido() {
        return itemPedido;
    }

    public void setItemPedido(ItemPedido itemPedido) {
        this.itemPedido = itemPedido;
    }

    public BigDecimal getValorTotal() {
        return valorTotal;
    }

    public void setValorTotal(BigDecimal valorTotal) {
        this.valorTotal = valorTotal;
    }

    public BigDecimal getValorUnitario() {
        return valorUnitario;
    }

    public void setValorUnitario(BigDecimal valorUnitario) {
        this.valorUnitario = valorUnitario;
    }

    public SituacaoRegistro getSituacaoRegistro() {
        return situacaoRegistro == null ? null : SituacaoRegistro.fromId(situacaoRegistro);
    }

    public void setSituacaoRegistro(SituacaoRegistro situacaoRegistro) {
        this.situacaoRegistro = situacaoRegistro == null ? null : situacaoRegistro.getId();
    }

    public Boolean getNaoEntregue() {
        return naoEntregue;
    }

    public void setNaoEntregue(Boolean naoEntregue) {
        this.naoEntregue = naoEntregue;
    }

    public MovimentoEstoque getMovimentoEstoque() {
        return movimentoEstoque;
    }

    public void setMovimentoEstoque(MovimentoEstoque movimentoEstoque) {
        this.movimentoEstoque = movimentoEstoque;
    }

    public Date getLastModifiedDate() {
        return lastModifiedDate;
    }

    public void setLastModifiedDate(Date lastModifiedDate) {
        this.lastModifiedDate = lastModifiedDate;
    }

    public String getLastModifiedBy() {
        return lastModifiedBy;
    }

    public void setLastModifiedBy(String lastModifiedBy) {
        this.lastModifiedBy = lastModifiedBy;
    }

    public Date getCreatedDate() {
        return createdDate;
    }

    public void setCreatedDate(Date createdDate) {
        this.createdDate = createdDate;
    }

    public String getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(String createdBy) {
        this.createdBy = createdBy;
    }

    public Integer getVersion() {
        return version;
    }

    public void setVersion(Integer version) {
        this.version = version;
    }

    public void setQuantidadeInformada(Double quantidadeInformada) {
        this.quantidadeInformada = quantidadeInformada;
    }

    public Double getQuantidadeInformada() {
        return quantidadeInformada;
    }

    public Item getItem() {
        return item;
    }

    public void setItem(Item item) {
        this.item = item;
    }

    public Pedido getPedido() {
        return pedido;
    }

    public void setPedido(Pedido pedido) {
        this.pedido = pedido;
    }

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }
}