package br.inf.mfp.gestaoadministrativa.screen.cadastros.item;

import br.inf.mfp.gestaoadministrativa.app.GrupoEconomicoBean;
import br.inf.mfp.gestaoadministrativa.entity.GrupoEconomico;
import br.inf.mfp.gestaoadministrativa.entity.Item;
import br.inf.mfp.gestaoadministrativa.screen.relkardexitem.RelKardexItem;
import io.jmix.ui.Notifications;
import io.jmix.ui.ScreenBuilders;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.ButtonsPanel;
import io.jmix.ui.component.Filter;
import io.jmix.ui.component.Table;
import io.jmix.ui.model.CollectionLoader;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

@UiController("ga_Item.browse")
@UiDescriptor("item-browse.xml")
@LookupComponent("itemsTable")
public class ItemBrowse extends StandardLookup<Item> {

    private GrupoEconomico filtroGe;

    @Autowired
    private Notifications notifications;
    @Autowired
    private GrupoEconomicoBean grupoEconomicoBean;
    @Autowired
    private ButtonsPanel buttonsPanel;
    @Autowired
    private Filter filter;
    @Autowired
    private Table<Item> itemsTable;
    @Autowired
    private ScreenBuilders screenBuilders;



    @Subscribe(id = "itemsDl", target = Target.DATA_LOADER)
    public void onItemsDlPreLoad(CollectionLoader.PreLoadEvent<Item> event) {
        if (filtroGe == null) {
            try {
                filtroGe = grupoEconomicoBean.retornarGE();
            } catch (Exception ee) {
                filtroGe = null;
            }
            if (filtroGe == null) {
                notifications.create(Notifications.NotificationType.ERROR).withCaption("Usuário sem Grupo Econômico").show();
                event.preventLoad();
                buttonsPanel.setEnabled(false);
                filter.setEnabled(false);
            }
        }
    }

    @Subscribe("extrato")
    public void onExtratoClick(Button.ClickEvent event) {

        if( itemsTable.getSingleSelected() != null) {

            try {

                RelKardexItem rel = screenBuilders.screen(this).withScreenClass(RelKardexItem.class)
                        .withOpenMode(OpenMode.DIALOG).build();
                        rel.setvItem(itemsTable.getSingleSelected());

                        rel.show();

            } catch (Exception ee) {
                notifications.create().withType(Notifications.NotificationType.ERROR)
                        .withCaption("Erro ao acessar a tela do Relatório")
                        .withDescription(ee.getMessage())
                        .show();
            }

        }
    }

    @Install(to = "itemsTable.extrato", subject = "enabledRule")
    private boolean itemsTableExtratoEnabledRule() {
        return  ( itemsTable.getSingleSelected() != null) ;
    }

}