package br.inf.mfp.gestaoadministrativa.screen.movimento.operacaoestoque;

import br.inf.mfp.gestaoadministrativa.app.GrupoEconomicoBean;
import br.inf.mfp.gestaoadministrativa.app.gaException;
import br.inf.mfp.gestaoadministrativa.entity.*;
import io.jmix.core.DataManager;
import io.jmix.ui.Notifications;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.Table;
import io.jmix.ui.icon.JmixIcon;
import io.jmix.ui.model.CollectionLoader;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

@UiController("ga_OperacaoEstoque.browse")
@UiDescriptor("operacao-estoque-browse.xml")
@LookupComponent("operacaoEstoquesTable")
public class OperacaoEstoqueBrowse extends StandardLookup<OperacaoEstoque> {

    //FIXME validar o grupo economico
    // - revisar o where dentro do DataLoader nos XML, acho que não é mais necessário
    @Autowired
    private Table<OperacaoEstoque> operacaoEstoquesTable;
    @Autowired
    private DataManager dataManager;
    @Autowired
    private Notifications notifications;
    @Autowired
    private CollectionLoader<OperacaoEstoque> operacaoEstoquesDl;
    @Autowired
    private GrupoEconomicoBean grupoEconomicoBean;

    @Subscribe(id = "operacaoEstoquesDl", target = Target.DATA_LOADER)
    public void onOperacaoEstoquesDlPreLoad(CollectionLoader.PreLoadEvent<OperacaoEstoque> event) {



    }

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {

        try {
            List<Empresa> filtroEmpresa = grupoEconomicoBean.retornarEmpresas();
            operacaoEstoquesDl.setParameter("empresa", filtroEmpresa);
            operacaoEstoquesDl.load();

        } catch (
                gaException e) {
            notifications.create(Notifications.NotificationType.ERROR).withCaption("Erro ao Carregar Grupo Economico").withDescription(e.getMessage()).show();
        }
    }


    @Install(to = "operacaoEstoquesTable.confirmar", subject = "enabledRule")
    private boolean operacaoEstoquesTableConfirmarEnabledRule() {
        if (operacaoEstoquesTable.getSingleSelected() != null) {
            return operacaoEstoquesTable.getSingleSelected().getSituacaoOperacao().equals(SituacaoOperacao.PENDENTE);
        }
        return false;

    }

    @Install(to = "operacaoEstoquesTable.retornar", subject = "enabledRule")
    private boolean operacaoEstoquesTableRetornarEnabledRule() {
        if (operacaoEstoquesTable.getSingleSelected() != null) {
            return operacaoEstoquesTable.getSingleSelected().getSituacaoOperacao().equals(SituacaoOperacao.CONFIRMADO);
        }
        return false;
    }

    @Subscribe("confirmarBtn")
    public void onConfirmarBtnClick(Button.ClickEvent event) {
        if (operacaoEstoquesTable.getSingleSelected() != null) {

            try {
                OperacaoEstoque oe = operacaoEstoquesTable.getSingleSelected();
                oe.setSituacaoOperacao(SituacaoOperacao.CONFIRMADO);
                dataManager.save(oe);

                notifications.create().withType(Notifications.NotificationType.HUMANIZED).withCaption("Operação realizada com sucesso").show();

            } catch (Exception ee) {
                notifications.create().withType(Notifications.NotificationType.ERROR).withCaption("Erro ao mudar a situação").withDescription(ee.getMessage()).show();
            }
            operacaoEstoquesDl.load();

        }

    }

    @Subscribe("retornar")
    public void onRetornarClick(Button.ClickEvent event) {
        if (operacaoEstoquesTable.getSingleSelected() != null) {

            try {
                OperacaoEstoque oe = operacaoEstoquesTable.getSingleSelected();
                oe.setSituacaoOperacao(SituacaoOperacao.PENDENTE);
                dataManager.save(oe);

                notifications.create().withType(Notifications.NotificationType.HUMANIZED).withCaption("Operação realizada com sucesso").show();

                operacaoEstoquesDl.load();
            } catch (Exception ee) {
                notifications.create().withType(Notifications.NotificationType.ERROR).withCaption("Erro ao mudar a situação").withDescription(ee.getMessage()).show();
            }

        }
    }

    @Install(to = "operacaoEstoquesTable.remove", subject = "enabledRule")
    private boolean operacaoEstoquesTableRemoveEnabledRule() {
        if (operacaoEstoquesTable.getSingleSelected() != null) {
            return operacaoEstoquesTable.getSingleSelected().getSituacaoOperacao().equals(SituacaoOperacao.PENDENTE);
        }
        return false;
    }

    @Install(to = "operacaoEstoquesTable.edit", subject = "enabledRule")
    private boolean operacaoEstoquesTableEditEnabledRule() {
        if (operacaoEstoquesTable.getSingleSelected() != null) {
            return operacaoEstoquesTable.getSingleSelected().getSituacaoOperacao().equals(SituacaoOperacao.PENDENTE);
        }
        return false;
    }

    @Install(to = "operacaoEstoquesTable", subject = "iconProvider")
    private String operacaoEstoquesTableIconProvider(OperacaoEstoque operacaoEstoque) {
        if (operacaoEstoque.getSituacaoOperacao().equals(SituacaoOperacao.CONFIRMADO))
            return JmixIcon.OK.source();
        else if (operacaoEstoque.getSituacaoOperacao().equals(SituacaoOperacao.PENDENTE))
            return JmixIcon.UNLINK.source();
        else if (operacaoEstoque.getSituacaoOperacao().equals(SituacaoOperacao.CANCELADO))
            return JmixIcon.CANCEL.source();
        //else if (pedido.getSituacaoPedido().equals(SituacaoPedido.PENDENTE))
            //return JmixIcon.EDIT_ACTION.source();
        else
            return null;

    }
}