package br.inf.mfp.gestaoadministrativa.screen.movimento.pedido;

import br.inf.mfp.gestaoadministrativa.app.GrupoEconomicoBean;
import br.inf.mfp.gestaoadministrativa.app.PedidoBean;
import br.inf.mfp.gestaoadministrativa.entity.GrupoEconomico;
import br.inf.mfp.gestaoadministrativa.entity.Pedido;
import br.inf.mfp.gestaoadministrativa.entity.SituacaoPedido;
import io.jmix.core.AccessManager;
import io.jmix.core.accesscontext.SpecificOperationAccessContext;
import io.jmix.reports.entity.ReportOutputType;
import io.jmix.reportsui.runner.ParametersDialogShowMode;
import io.jmix.reportsui.runner.UiReportRunner;
import io.jmix.ui.Notifications;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.ButtonsPanel;
import io.jmix.ui.component.Filter;
import io.jmix.ui.component.Table;
import io.jmix.ui.icon.JmixIcon;
import io.jmix.ui.model.CollectionLoader;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

@UiController("ga_Pedido.browse")
@UiDescriptor("pedido-browse.xml")
@LookupComponent("pedidoesTable")
public class PedidoBrowse extends StandardLookup<Pedido> {

    @Autowired
    private CollectionLoader<Pedido> pedidoesDl;
    @Autowired
    private Notifications notifications;
    @Autowired
    private Table<Pedido> pedidoesTable;
    @Autowired
    private PedidoBean pedidoBean;

    boolean AcessoCancelar = false;
    @Autowired
    private AccessManager accessManager;

    private GrupoEconomico filtroGe;
    @Autowired
    private GrupoEconomicoBean grupoEconomicoBean;
    @Autowired
    private ButtonsPanel buttonsPanel;
    @Autowired
    private Filter filter;
    @Autowired
    private UiReportRunner uiReportRunner;

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {

            SpecificOperationAccessContext acessContext = new SpecificOperationAccessContext("cancelar");
            accessManager.applyRegisteredConstraints(acessContext);
            if (acessContext.isPermitted()){
                //cancelarBtn.setEnabled(false);
                AcessoCancelar = true;
            }

    }

    @Install(to = "pedidoesTable.confirmarPedido", subject = "enabledRule")
    private boolean pedidoesTableLiberarPedidoEnabledRule() {
        if (pedidoesTable.getSingleSelected() == null) return false;
        return pedidoesTable.getSingleSelected().getSituacaoPedido().equals(SituacaoPedido.PENDENTE);
    }

    @Subscribe("confirmarBtn")
    public void onLiberarBtnClick(Button.ClickEvent event) {

        if (pedidoesTable.getSingleSelected() == null) return;

        try {

            if (pedidoBean.AlterarPedido(pedidoesTable.getSingleSelected(), SituacaoPedido.PENDENTE, SituacaoPedido.CONFIRMADO))
                pedidoesDl.load();

            notifications.create(Notifications.NotificationType.HUMANIZED).withCaption("Pedido Confirmado").show();

        } catch (Exception ee) {
            notifications.create(Notifications.NotificationType.ERROR).withCaption("Erro ao Confirmar o Pedido").withDescription(ee.getMessage()).show();
        }
    }

    @Install(to = "pedidoesTable.desconfirmarPedido", subject = "enabledRule")
    private boolean pedidoesTableCancelarLiberacaoEnabledRule() {
        if (pedidoesTable.getSingleSelected() == null) return false;
        return pedidoesTable.getSingleSelected().getSituacaoPedido().equals(SituacaoPedido.CONFIRMADO);
    }

    @Install(to = "pedidoesTable.edit", subject = "enabledRule")
    private boolean pedidoesTableEditEnabledRule() {
        if (pedidoesTable.getSingleSelected() == null) return false;
        return pedidoesTable.getSingleSelected().getSituacaoPedido().equals(SituacaoPedido.PENDENTE);
    }

    @Install(to = "pedidoesTable.cancelarPedido", subject = "enabledRule")
    private boolean pedidoesTableCancelarEnabledRule() {
        if (pedidoesTable.getSingleSelected() == null) return false;
        return pedidoesTable.getSingleSelected().getSituacaoPedido().equals(SituacaoPedido.PENDENTE) && AcessoCancelar;
    }

    @Subscribe("cancelarBtn")
    public void onCancelarBtnClick(Button.ClickEvent event) {
        if (pedidoesTable.getSingleSelected() == null) return;

        try {

            if (pedidoBean.AlterarPedido(pedidoesTable.getSingleSelected(), SituacaoPedido.PENDENTE, SituacaoPedido.CANCELADO))
                pedidoesDl.load();

            notifications.create(Notifications.NotificationType.HUMANIZED).withCaption("Pedido Cancelado").show();

        } catch (Exception ee) {
            notifications.create(Notifications.NotificationType.ERROR).withCaption("Erro ao cancelar o Pedido").withDescription(ee.getMessage()).show();
        }
    }

    @Subscribe("desconfirmarBtn")
    public void onCancelarLiberacaoBtnClick(Button.ClickEvent event) {
        if (pedidoesTable.getSingleSelected() == null) return;

        try {

            if (pedidoBean.AlterarPedido(pedidoesTable.getSingleSelected(), SituacaoPedido.CONFIRMADO, SituacaoPedido.PENDENTE))
                pedidoesDl.load();

            notifications.create(Notifications.NotificationType.HUMANIZED).withCaption("Pedido pendente").show();

        } catch (Exception ee) {
            notifications.create(Notifications.NotificationType.ERROR).withCaption("Erro ao desconfirmar o Pedido para pendente").withDescription(ee.getMessage()).show();
        }
    }

    @Install(to = "pedidoesTable.revisarRecebimento", subject = "enabledRule")
    private boolean pedidoesTableRevisarRecebimentoEnabledRule() {
        if (pedidoesTable.getSingleSelected() == null) return false;
        return pedidoesTable.getSingleSelected().getSituacaoPedido().equals(SituacaoPedido.RECEBIDO) && AcessoCancelar;

    }

    @Subscribe("revisarBtn")
    public void onRevisarBtnClick(Button.ClickEvent event) {
        if (pedidoesTable.getSingleSelected() == null) return;

        try {

            if (pedidoBean.AlterarPedido(pedidoesTable.getSingleSelected(), SituacaoPedido.RECEBIDO, SituacaoPedido.CONFIRMADO))
                pedidoesDl.load();

            notifications.create(Notifications.NotificationType.HUMANIZED).withCaption("Pedido liberado para Recebimento").show();

        } catch (Exception ee) {
            notifications.create(Notifications.NotificationType.ERROR).withCaption("Erro ao liberar o Pedido para Recebimento").withDescription(ee.getMessage()).show();
        }
    }

    @Install(to = "pedidoesTable", subject = "iconProvider")
    private String pedidoesTableIconProvider(Pedido pedido) {
        if (pedido.getSituacaoPedido().equals(SituacaoPedido.CONFIRMADO))
            return JmixIcon.CUBES.source();
        else if (pedido.getSituacaoPedido().equals(SituacaoPedido.RECEBIDO))
            return JmixIcon.OK.source();
        else if (pedido.getSituacaoPedido().equals(SituacaoPedido.CANCELADO))
            return JmixIcon.CANCEL.source();
        else if (pedido.getSituacaoPedido().equals(SituacaoPedido.PENDENTE))
            return JmixIcon.EDIT_ACTION.source();


        else
            return null;
    }

    @Subscribe(id = "pedidoesDl", target = Target.DATA_LOADER)
    public void onPedidoesDlPreLoad(CollectionLoader.PreLoadEvent<Pedido> event) {
        if (filtroGe == null) {
            try {
                filtroGe = grupoEconomicoBean.retornarGE();
            } catch (Exception ee) {
                filtroGe = null;
            }
            if (filtroGe == null) {
                notifications.create(Notifications.NotificationType.ERROR).withCaption("Usuário sem Grupo Econômico").show();
                event.preventLoad();
                buttonsPanel.setEnabled(false);
                filter.setEnabled(false);
            }
        }
    }

    @Subscribe("detalhes")
    public void onDetalhesClick(Button.ClickEvent event) {
        if( pedidoesTable.getSingleSelected() != null) {

            try {

                uiReportRunner.byReportCode("002")
                        .addParam("pedido", pedidoesTable.getSingleSelected() )
                        .withOutputType(ReportOutputType.PDF)
                        .withParametersDialogShowMode(ParametersDialogShowMode.NO)
                        .runAndShow();


            } catch (Exception e) {
                notifications.create().withType(Notifications.NotificationType.ERROR)
                        .withCaption("Erro ao executar o Relatório")
                        .withDescription(e.getMessage())
                        .show();
            }

        }
    }

    @Install(to = "pedidoesTable.detalhesprn", subject = "enabledRule")
    private boolean pedidoesTableDetalhesprnEnabledRule() {
        return  (  pedidoesTable.getSingleSelected() != null) ;
    }

}