package br.inf.mfp.gestaoadministrativa.screen.multiempresa;

import br.inf.mfp.gestaoadministrativa.app.EmpresaBean;
import br.inf.mfp.gestaoadministrativa.app.gaException;
import br.inf.mfp.gestaoadministrativa.entity.Situacao;
import br.inf.mfp.gestaoadministrativa.entity.User;
import br.inf.mfp.gestaoadministrativa.entity.UsuarioEmpresa;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.ui.Notifications;
import io.jmix.ui.UiComponents;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.CheckBox;
import io.jmix.ui.component.Component;
import io.jmix.ui.component.Table;
import io.jmix.ui.component.data.TableItems;
import io.jmix.ui.model.CollectionContainer;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;

import java.time.Instant;
import java.util.Date;
import java.util.Objects;

@UiController("ga_MultiEmpresa")
@UiDescriptor("multi-empresa.xml")
public class MultiEmpresa extends Screen {

    @Autowired
    private EmpresaBean empresaBean;

    @Autowired
    private CurrentAuthentication usuarioAut;
    @Autowired
    private UiComponents uiComponents;
    @Autowired
    private CollectionContainer<UsuarioEmpresa> empresaDc;
    @Autowired
    private Notifications notifications;
    @Autowired
    private Table<UsuarioEmpresa> usuariosTable;

    @Subscribe("btnFechar")
    public void onBtnFecharClick(Button.ClickEvent event) {
        this.closeWithDefaultAction();
    }

    private String loginUsuario;
    @Subscribe
    public void onAfterShow(AfterShowEvent event) {

        try {
            empresaBean.loginUsuario = loginUsuario;
            empresaDc.setItems(empresaBean.CargaEmpresaUsuario());
        } catch (gaException e) {
            notifications.create(Notifications.NotificationType.ERROR).withCaption("Erro ao Efetuar a carga").withDescription(e.getMessage()).show();

        }

    }

    @Install(to = "usuariosTable.ativo", subject = "columnGenerator")
    private Component usuariosTableAtivoColumnGenerator(UsuarioEmpresa usuarioEmpresa) {
        CheckBox chk = uiComponents.create(CheckBox.class);
        chk.setValue(usuarioEmpresa.getSituacao().equals(Situacao.ATIVO));
        chk .setEditable(true);
        chk.setEnabled(true);

        chk.addValueChangeListener(booleanValueChangeEvent -> {
            usuarioEmpresa.setSituacao(Objects.equals(booleanValueChangeEvent.getValue(), true) ? Situacao.ATIVO : Situacao.BLOQUEADO);
        });

        return chk;
    }

    @Subscribe("btnSelTodos")
    public void onBtnSelTodosClick(Button.ClickEvent event) {
        TableItems<UsuarioEmpresa> listaItens = usuariosTable.getItems();
        if (listaItens != null) {
            listaItens.getItems().forEach(usuarioEmpresa -> {
                usuarioEmpresa.setSituacao(Situacao.ATIVO);
            });
        }
    }

    @Subscribe("btnInvSel")
    public void onBtnInvSelClick(Button.ClickEvent event) {
        TableItems<UsuarioEmpresa> listaItens = usuariosTable.getItems();
        if (listaItens != null) {
            listaItens.getItems().forEach(usuarioEmpresa -> {
                usuarioEmpresa.setSituacao(usuarioEmpresa.getSituacao().equals(Situacao.ATIVO) ? Situacao.BLOQUEADO : Situacao.ATIVO);
            });
        }
    }

    @Subscribe("btnConfirmar")
    public void onBtnConfirmarClick(Button.ClickEvent event) {

        try {

            empresaBean.SalvarUsuariosEmpresa(empresaDc.getItems());

            notifications.create(Notifications.NotificationType.HUMANIZED).withCaption("Alterações salvas").show();
            this.closeWithDefaultAction();

        } catch (Exception e) {
            notifications.create(Notifications.NotificationType.ERROR).withCaption("Erro ao Salvar os Dados").withDescription("Erro => " + e.getMessage()).show();
        }


    }


    public void setLoginUsuario(String loginUsuario) {
        this.loginUsuario = loginUsuario;
    }
}