package br.inf.mfp.gestaoadministrativa.screen.relkardexitem;

import br.inf.mfp.gestaoadministrativa.app.GrupoEconomicoBean;
import br.inf.mfp.gestaoadministrativa.app.gaException;
import br.inf.mfp.gestaoadministrativa.entity.Empresa;
import br.inf.mfp.gestaoadministrativa.entity.GrupoEconomico;
import br.inf.mfp.gestaoadministrativa.entity.Item;
import io.jmix.reports.entity.ReportOutputType;
import io.jmix.reportsui.runner.ParametersDialogShowMode;
import io.jmix.reportsui.runner.UiReportRunner;
import io.jmix.ui.Notifications;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.DateField;
import io.jmix.ui.component.EntityComboBox;
import io.jmix.ui.component.GroupBoxLayout;
import io.jmix.ui.model.CollectionLoader;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;

@UiController("ga_RelKardexItem")
@UiDescriptor("rel-kardex-item.xml")
public class RelKardexItem extends Screen  implements ScreenOptions {
    @Autowired
    private CollectionLoader<Empresa> empresasDl;
    @Autowired
    private CollectionLoader<Item> itemsDl;
    @Autowired
    private Notifications notifications;
    @Autowired
    private EntityComboBox<Empresa> empresaFiltro;
    @Autowired
    private EntityComboBox<Item> itemFiltro;
    @Autowired
    private DateField<Date> dataIni;
    @Autowired
    private DateField<Date> datafinal;
    @Autowired
    private UiReportRunner uiReportRunner;
    @Autowired
    private GroupBoxLayout gr001;
    @Autowired
    private GrupoEconomicoBean grupoEconomicoBean;

    private Item vItem;

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {

        GrupoEconomico filtroGe;
        try {
                filtroGe = grupoEconomicoBean.retornarGE();

            } catch (Exception ee) {
                filtroGe = null;
            }

            if (filtroGe == null) {
                notifications.create(Notifications.NotificationType.ERROR).withCaption("Usuário sem Grupo Econômico").show();
                gr001.setEnabled(false);
            }else{
                empresasDl.load();
                itemsDl.load();
                if(vItem != null){
                    itemFiltro.setValue(vItem);
                }
    }


    }

    @Subscribe("closeBtn")
    public void onCloseBtnClick(Button.ClickEvent event) {
        this.close(WINDOW_CLOSE_ACTION);
    }

    @Subscribe("imprimirBtn")
    public void onImprimirBtnClick(Button.ClickEvent event) {

        try{
            if(empresaFiltro.getValue() == null)
                throw new gaException( "A Empresa deve ser selecionada.");
            if( itemFiltro.getValue() == null)
                throw new gaException( "O Item deve ser selecionada.");
            if( dataIni.getValue() == null)
                throw new gaException( "A data Inicial deve ser informada.");

            if(datafinal.getValue() == null)
                throw new gaException( "A data Final deve ser informada.");

            if (dataIni.getValue().after(datafinal.getValue()))
                throw new gaException( "A data Final deve ser igual ou maior que a data Inicial.");

        }catch (Exception ee){
            notifications.create().withType(Notifications.NotificationType.ERROR).withCaption("Erro ao Imprimir").withDescription(ee.getMessage()).show();
            return;
        }

        try{

            uiReportRunner.byReportCode("001")
                    .addParam("data_inicio", dataIni.getValue())
                    .addParam( "data_final",datafinal.getValue())
                    .addParam("item_selecionado", itemFiltro.getValue() )
                    .addParam("empresa", empresaFiltro.getValue())
                    .withOutputType(ReportOutputType.PDF)
                    .withParametersDialogShowMode(ParametersDialogShowMode.NO)
                    .runAndShow();

        }catch (Exception ee){
            notifications.create().withType(Notifications.NotificationType.ERROR).withCaption("Erro ao processar o relatório").withDescription(ee.getMessage()).show();
        }


    }


    public void setvItem(Item vItem) {
        this.vItem = vItem;
    }
}