package br.inf.mfp.gestaoadministrativa.screen.retornopedidousuario;

import br.inf.mfp.gestaoadministrativa.app.GrupoEconomicoBean;
import br.inf.mfp.gestaoadministrativa.app.PedidoBean;
import br.inf.mfp.gestaoadministrativa.app.gaException;
import br.inf.mfp.gestaoadministrativa.entity.*;
import br.inf.mfp.gestaoadministrativa.screen.movimento.retornopedido.RetornoPedidoEdit;
import io.jmix.core.DataManager;
import io.jmix.ui.Dialogs;
import io.jmix.ui.Notifications;
import io.jmix.ui.ScreenBuilders;
import io.jmix.ui.action.DialogAction;
import io.jmix.ui.component.*;
import io.jmix.ui.icon.JmixIcon;
import io.jmix.ui.model.*;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.List;

@UiController("ga_RetornoPedidoUsuario")
@UiDescriptor("retorno-pedido-usuario.xml")
public class RetornoPedidoUsuario extends Screen {
    @Autowired
    private GrupoEconomicoBean grupoEconomicoBean;

    private GrupoEconomico grupoEconomico;
    @Autowired
    private Notifications notifications;
    @Autowired
    private InstanceLoader<Pedido> pedidoLc;
    @Autowired
    private TextField<Integer> numeroField;
    @Autowired
    private InstanceContainer<Pedido> pedidoDc;
    @Autowired
    private PedidoBean pedidoBean;
    @Autowired
    private Button btnPesquisar;
    @Autowired
    private CollectionPropertyContainer<RetornoPedido> retornoPedidoDc;
    @Autowired
    private DataManager dataManager;
    @Autowired
    private GroupBoxLayout grItem;
    @Autowired
    private DataContext dataContext;
    @Autowired
    private Button salvarBtn;
    @Autowired
    private Table<RetornoPedido> retornoPedidoesTable;
    @Autowired
    private EntityComboBox<Item> itemCbo;
    @Autowired
    private CheckBox naoEntregueField;
    @Autowired
    private Dialogs dialogs;
    @Autowired
    private ScreenBuilders screenBuilders;
    @Autowired
    private Button confirmarBtn;
    @Autowired
    private Button adicionarBtn;
    @Autowired
    private TextField<Double> qtdItemField;

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {

        try {
            grupoEconomico = grupoEconomicoBean.retornarGE();

            pedidoLc.setParameter("numero", -1);

            grItem.setVisible(false);

        } catch (gaException e) {
            notifications.create(Notifications.NotificationType.ERROR).withCaption("Erro ao Carregar Grupo Economico").withDescription(e.getMessage()).show();
        }

    }

    @Subscribe("btnPesquisar")
    public void onBtnPesquisarClick(Button.ClickEvent event) {

        CargaPedido();

    }

    private void CargaPedido() {

        try {

            pedidoDc.setItem(pedidoBean.consultaPedido(numeroField.getValue(), grupoEconomico.getId()));

            if (pedidoDc.getItem().getRetornoPedido() != null) {
                List<RetornoPedido> retorno = new ArrayList<>();

                if (pedidoDc.getItem().getRetornoPedido().size() > 0) {
                    retorno = pedidoDc.getItem().getRetornoPedido();
                }

                if (pedidoDc.getItem().getItemPedido().size() > 0) {
                    List<RetornoPedido> finalRetorno = retorno;
                    pedidoDc.getItem().getItemPedido().forEach(itemPedido -> {
                        boolean achouItem = false;

                        for (RetornoPedido retornoPedido : finalRetorno) {
                            if (retornoPedido.getItem().getId().equals(itemPedido.getItem().getId())) {
                                achouItem = true;
                                break;
                            }
                        }

                        if (!achouItem) {
                            RetornoPedido item = dataContext.create(RetornoPedido.class);
                            item.setItem(itemPedido.getItem());
                            item.setPedido(itemPedido.getPedido());
                            item.setNaoEntregue(false);
                            item.setQuantidadeInformada(0d);

                            finalRetorno.add(item);
                        }
                    });

                    pedidoDc.getItem().setRetornoPedido(finalRetorno);
                }
            }

            retornoPedidoDc.setItems(pedidoDc.getItem().getRetornoPedido());

            numeroField.setEditable(false);
            btnPesquisar.setEnabled(false);
            grItem.setVisible(true);

            if(pedidoDc.getItem().getSituacaoPedido().equals(SituacaoPedido.CONFIRMADO)) {
                salvarBtn.setEnabled(true);
                confirmarBtn.setEnabled(true);
                grItem.setEnabled(true);

            }
            else
            {
                salvarBtn.setEnabled(false);
                confirmarBtn.setEnabled(false);
                grItem.setEnabled(false);
                notifications.create(Notifications.NotificationType.SYSTEM).withCaption("Pedido não disponível para recebimento").show();
            }
        } catch (gaException e) {
            notifications.create(Notifications.NotificationType.ERROR).withCaption("Erro ao Pesquisar").withDescription(e.getMessage()).show();
            btnPesquisar.setEnabled(true);
            numeroField.setEditable(true);
            salvarBtn.setEnabled(false);
        }

    }

    @Subscribe("btnlimpar")
    public void onBtnlimparClick(Button.ClickEvent event) {
        pedidoDc.setItem(null);
        btnPesquisar.setEnabled(true);
        salvarBtn.setEnabled(false);
        numeroField.setEditable(true);
        grItem.setVisible(false);
        numeroField.focus();

    }

    @Subscribe("salvarBtn")
    public void onSalvarBtnClick(Button.ClickEvent event) {

        dialogs.createOptionDialog()
                .withCaption("Salvar dados Recebimento")
                .withMessage("Deseja realmente salvar as informações?")
                .withActions(
                        new DialogAction(DialogAction.Type.YES)
                                .withHandler(e ->
                                        doSalvarRecebimento(1)
                                ),
                        new DialogAction(DialogAction.Type.CANCEL))
                .show();


    }

    private void doSalvarRecebimento(int acao) {
        dataManager.save(pedidoDc.getItem());

        if (acao == 1)
            CargaPedido();
        else
            onBtnlimparClick(null);
    }

    @Install(to = "retornoPedidoesTable", subject = "iconProvider")
    private String retornoPedidoesTableIconProvider(RetornoPedido retornoPedido) {
        if (retornoPedido.getSituacaoRegistro().equals(SituacaoRegistro.INFORMADO))
            return JmixIcon.OK.source();
        else if (retornoPedido.getSituacaoRegistro().equals(SituacaoRegistro.EDITAR) || retornoPedido.getSituacaoRegistro().equals(SituacaoRegistro.EDITAR_SEM))
            return JmixIcon.EDIT_ACTION.source();
        else
            return null;

    }

    @Subscribe("numeroField")
    public void onNumeroFieldEnterPress(TextInputField.EnterPressEvent event) {
        onBtnPesquisarClick(null);
    }

    @Subscribe("retornoPedidoesTable")
    public void onRetornoPedidoesTableSelection(Table.SelectionEvent<RetornoPedido> event) {

        if (retornoPedidoesTable.getSingleSelected() == null)
            return;

        itemCbo.setValue(retornoPedidoesTable.getSingleSelected().getItem());
        qtdItemField.setValue(retornoPedidoesTable.getSingleSelected().getQuantidadeInformada());
        naoEntregueField.setValue(retornoPedidoesTable.getSingleSelected().getNaoEntregue());
        adicionarBtn.setEnabled(false);

        retornoPedidoesTable.getSingleSelected().setSituacaoRegistro(retornoPedidoesTable.getSingleSelected().getSituacaoRegistro().equals(SituacaoRegistro.PENDENTE) ? SituacaoRegistro.EDITAR_SEM : SituacaoRegistro.EDITAR);
        retornoPedidoesTable.setEnabled(false);
    }

    @Subscribe("editBtn")
    public void onEditBtnClick(Button.ClickEvent event) {
        if (retornoPedidoesTable.getSingleSelected() == null)
            return;

        List<RetornoPedido> pedido = pedidoDc.getItem().getRetornoPedido();

        for (int i = 0; i < pedido.size(); i++) {

            RetornoPedido retornoPedido = pedido.get(i);
            if (retornoPedido.getId().equals(retornoPedidoesTable.getSingleSelected().getId())) {
                retornoPedido.setNaoEntregue(naoEntregueField.getValue());
                retornoPedido.setQuantidadeInformada( qtdItemField.getValue());

                retornoPedido.setSituacaoRegistro(SituacaoRegistro.INFORMADO);
                pedidoDc.getItem().setRetornoPedido(pedido);
            }
        }

        retornoPedidoesTable.setEnabled(true);

        itemCbo.setValue(null);
        qtdItemField.setValue(null);
        naoEntregueField.setValue(null);
        adicionarBtn.setEnabled(true);

    }

    @Subscribe("LimparBtn")
    public void onLimparBtnClick(Button.ClickEvent event) {

        if (retornoPedidoesTable.getSingleSelected() == null)
            return;

        itemCbo.setValue(null);
        qtdItemField.setValue(0D);
        //qtdItemField.setValue(null);
        naoEntregueField.setValue(null);
        adicionarBtn.setEnabled(true);
        

        retornoPedidoesTable.getSingleSelected().setSituacaoRegistro(retornoPedidoesTable.getSingleSelected().getSituacaoRegistro().equals(SituacaoRegistro.EDITAR_SEM) ? SituacaoRegistro.PENDENTE : SituacaoRegistro.INFORMADO);
        retornoPedidoesTable.setEnabled(true);
    }

    @Subscribe("confirmarBtn")
    public void onConfirmarBtnClick(Button.ClickEvent event) {

        dialogs.createOptionDialog()
                .withCaption("Confirmar Recebimento")
                .withMessage("Deseja realmente confirmar o pedido como recebido?")
                .withActions(
                        new DialogAction(DialogAction.Type.OK)
                                .withHandler(e ->
                                        doConfirmarRecebimento()
                                ),
                        new DialogAction(DialogAction.Type.CANCEL))
                .show();

    }

    private void doConfirmarRecebimento() {
        pedidoDc.getItem().setSituacaoPedido(SituacaoPedido.RECEBIDO);
        doSalvarRecebimento(2);

    }

    @Subscribe("adicionarBtn")
    public void onAdicionarBtnClick(Button.ClickEvent event) {
        screenBuilders.editor(RetornoPedido.class, this)
                .newEntity()
                .withInitializer(retornoPedido -> {
                            retornoPedido.setPedido(pedidoDc.getItem());
                            retornoPedido.setSituacaoRegistro(SituacaoRegistro.INFORMADO);

                        }
                )
                .withScreenClass(RetornoPedidoEdit.class)
                .withOpenMode(OpenMode.DIALOG)
                .withAfterCloseListener(retornoPedidoEditAfterScreenCloseEvent -> {
                    if (retornoPedidoEditAfterScreenCloseEvent.closedWith(StandardOutcome.COMMIT)) {
                        RetornoPedido retornoPedido = retornoPedidoEditAfterScreenCloseEvent.getSource().getEditedEntity();
                        AdicionarRetorno(retornoPedido);
                    }
                })
                .build()
                .show();
    }

    private void AdicionarRetorno(RetornoPedido retornoPedido) {

        pedidoDc.getItem().getRetornoPedido().add(retornoPedido);

        retornoPedidoDc.setItems(pedidoDc.getItem().getRetornoPedido());

        notifications.create()
                .withType(Notifications.NotificationType.HUMANIZED)
                .withCaption("Item Adicionado com sucesso")
                .show();

    }

}