package br.inf.mfp.gestaoadministrativa.screen.traducao;

import br.inf.mfp.gestaoadministrativa.app.TraducaoBean;
import br.inf.mfp.gestaoadministrativa.entity.SituacaoTraducao;
import br.inf.mfp.gestaoadministrativa.entity.TraducaoLinhas;
import io.jmix.ui.Notifications;
import io.jmix.ui.component.*;
import io.jmix.ui.model.CollectionContainer;
import io.jmix.ui.model.CollectionPropertyContainer;
import io.jmix.ui.model.InstanceContainer;
import io.jmix.ui.screen.*;
import br.inf.mfp.gestaoadministrativa.entity.Traducao;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.stream.Collectors;

@UiController("ga_Traducao.edit")
@UiDescriptor("traducao-edit.xml")
@EditedEntityContainer("traducaoDc")
public class TraducaoEdit extends StandardEditor<Traducao> {
    @Autowired
    private Table<TraducaoLinhas> traducaoLinhasTable;
    @Autowired
    private TextField<String> conteudo;
    @Autowired
    private Button copiarBtn;
    @Autowired
    private Notifications notifications;
    @Autowired
    private CollectionPropertyContainer<TraducaoLinhas> traducaoLinhasDc;
    @Autowired
    private CheckBox apenasChk;
    @Autowired
    private Label<String> qtdReg;
    @Autowired
    private TraducaoBean traducaoBean;


    @Subscribe("traducaoLinhasTable")
    public void onTraducaoLinhasTableSelection(Table.SelectionEvent<TraducaoLinhas> event) {
        if (traducaoLinhasTable.getSingleSelected() != null) {

            conteudo.setValue(traducaoLinhasTable.getSingleSelected().getConteudoOriginal());
            copiarBtn.setEnabled(true);


            //clipTrigger.setInput( traducaoLinhasTable.getSingleSelected().getConteudoOriginal());

        }
    }

    @Subscribe("conteudo")
    public void onConteudoValueChange(HasValue.ValueChangeEvent event) {
        copiarBtn.setEnabled(event.getValue() != null);
    }

    @Subscribe(id = "traducaoLinhasDc", target = Target.DATA_CONTAINER)
    public void onTraducaoLinhasDcItemPropertyChange(InstanceContainer.ItemPropertyChangeEvent<TraducaoLinhas> event) {
        TraducaoLinhas linha = event.getItem();

        if (linha.getRetornoApi() != null) {
            linha.setSituacao(SituacaoTraducao.TRADUZIDO_API);
        }
        else if (linha.getConteudoTraduzido() != null && linha.getConteudoTraduzido().equals(linha.getConteudoOriginal())){
            linha.setSituacao(SituacaoTraducao.COPIADO);
        }
        else if (linha.getConteudoTraduzido() != null) {
            linha.setSituacao(SituacaoTraducao.TRADUZIDO);
        } else {
            linha.setSituacao(SituacaoTraducao.PENDENTE);
        }
        if (apenasChk.isChecked())
            FiltrarGrade(true);

    }

    @Subscribe("apenasChk")
    public void onApenasChkValueChange(HasValue.ValueChangeEvent<Boolean> event) {
        if (event.getValue() != null)
            FiltrarGrade(event.getValue());

    }

    @Subscribe(id = "traducaoLinhasDc", target = Target.DATA_CONTAINER)
    public void onTraducaoLinhasDcCollectionChange(CollectionContainer.CollectionChangeEvent<TraducaoLinhas> event) {
        if (traducaoLinhasTable.getItems() != null)
            qtdReg.setValue(String.valueOf(traducaoLinhasTable.getItems().size()));
    }

    private void FiltrarGrade(Boolean value) {

        if (value) {

            List<TraducaoLinhas> linhas = getEditedEntity().getTraducaoLinhas().stream().filter(traducaoLinhas ->
                    traducaoLinhas.getSituacao().equals(SituacaoTraducao.PENDENTE)).collect(Collectors.toList());

            traducaoLinhasDc.setDisconnectedItems(linhas);

        } else {
            traducaoLinhasDc.setDisconnectedItems(getEditedEntity().getTraducaoLinhas());
        }
    }

    @Subscribe("traduzirBtn")
    public void onTraduzirBtnClick(Button.ClickEvent event) {
        try{

            if (traducaoLinhasTable.getSingleSelected() != null) {

                String retorno = traducaoBean.ChamarTraducao(traducaoLinhasTable.getSingleSelected().getConteudoOriginal());

                if (!retorno.isEmpty()){
                    traducaoLinhasTable.getSingleSelected().setConteudoTraduzido(retorno);
                    traducaoLinhasTable.getSingleSelected().setRetornoApi(retorno);
                    //traducaoLinhasTable.getSingleSelected().setSituacao(SituacaoTraducao.TRADUZIDO_API);
                }

                //notifications.create(Notifications.NotificationType.HUMANIZED).withCaption(retorno);
            }

        }
        catch (Exception e){
            notifications.create(Notifications.NotificationType.ERROR).withCaption("Erro ao Traduzir").withDescription(e.getMessage()).show();
        }
    }

    @Subscribe("manterBtn")
    public void onManterBtnClick(Button.ClickEvent event) {
        if (traducaoLinhasTable.getSingleSelected() != null)
            traducaoLinhasTable.getSingleSelected().setConteudoTraduzido(traducaoLinhasTable.getSingleSelected().getConteudoOriginal());
    }


}